
import unittest
import asyncio
import pathlib


class NTDSParse:
	async def runner(self, parser, out_q):
		_, err = await parser.run()
		if err is not None:
			await out_q.put((None, None, err, True))

	async def parser_test(self, crypolibname):
		try:
			import unicrypto
			unicrypto.use_library(crypolibname)
			from aesedb.examples.ntdsparse import NTDSParserConsole
			from .knownvalues import knowngood
			basepath = pathlib.Path(__file__).parent.resolve()
			ntdsfile = str(basepath.joinpath('ntds.dit'))
			systemfile = str(basepath.joinpath('SYSTEM'))

			lm_deleted = {}
			nt_deleted = {}

			out_q = asyncio.Queue()
			parser = NTDSParserConsole(systemfile, ntdsfile, show_progress = True, outfile = None, ext_result_q=out_q, count_total = False)
			x = asyncio.create_task(self.runner(parser, out_q))
			while True:
				usersecret, total, err, eof = await out_q.get()
				
				if err is not None:
					raise err
				if eof is True:
					break
				if usersecret is None:
					continue
				
				for secret in str(usersecret).split('\r\n'):
					ht, *t = secret.split(':')
					if ht == 'ntlm':
						domain, user, rid, sid, lm, nt, lastset_or_history = t
						if lm not in lm_deleted:
							del knowngood['lmhashes'][lm]
							lm_deleted[lm] = 1
						if nt not in nt_deleted:
							del knowngood['nthashes'][nt]
							nt_deleted[nt] = 1
					elif ht == 'kerberos':
						domain, user, sid, kt, secret = tuple(t)
					elif ht == 'cleartext':
						domain, user, sid, secret = tuple(t)
				
			if len(knowngood['lmhashes']) > 0:
				raise Exception('LM hashes remaining: %s' % len(knowngood['lmhashes']))
				
			if len(knowngood['nthashes']) > 0:
				raise Exception('NT hashes remaining: %s' % len(knowngood['nthashes']))
			with open('remaining_nt.txt', 'w', newline = '') as f:
				for x in knowngood['nthashes']:
					f.write(x + '\r\n')
			return True, None
		except Exception as e:
			print(e)
			return False, e


#class pure(NTDSParse, unittest.IsolatedAsyncioTestCase):	
#	async def test_parser(self):
#		_, err = await self.parser_test('pure')
#		if err is not None:
#			raise err
#
#class Crypto(NTDSParse, unittest.IsolatedAsyncioTestCase):
#	async def test_parser(self):
#		_, err = await self.parser_test('pure')
#		if err is not None:
#			raise err
#
#class pyCryptodome(NTDSParse, unittest.IsolatedAsyncioTestCase):
#	async def test_parser(self):
#		_, err = await self.parser_test('pure')
#		if err is not None:
#			raise err
#
#class MBEDTLS(NTDSParse, unittest.IsolatedAsyncioTestCase):
#	async def test_parser(self):
#		_, err = await self.parser_test('pure')
#		if err is not None:
#			raise err

class cryptography(NTDSParse, unittest.IsolatedAsyncioTestCase):
	async def test_parser(self):
		_, err = await self.parser_test('pure')
		if err is not None:
			raise err

if __name__ == '__main__':
	unittest.main()


import io

class ESENT_SPACE_TREE_HEADER:
	def __init__(self):
		self.Unknown = None
	
	@staticmethod
	def from_bytes(data):
		return ESENT_SPACE_TREE_HEADER.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_SPACE_TREE_HEADER()
		res.Unknown = int.from_bytes(buff.read(8), byteorder='little', signed=False)
		return res



import asyncio
from tqdm import tqdm
from aesedb.eesentdb import ESENT_DB
from aesedb.utils.afile import AFile
from aesedb.security.ntds import NTDS
from aesedb.utils.systemhive import SYSTEM
from aiowinreg.ahive import AIOWinRegHive
from aesedb import logger

class NTDSParserConsole:
	def __init__(self, bootkey, ntdsfile, ignore_errors = True, with_history = True):
		self.bootkey = bootkey
		self.ntdsfile = ntdsfile
		self.ignore_errors = ignore_errors
		self.with_history = with_history

	async def get_total_rows(self):
		logger.debug('Fetching total row count...')
		file = self.ntdsfile
		if isinstance(self.ntdsfile, str):
			file = AFile(self.ntdsfile)

		db = ESENT_DB(file)
		_, err = await db.parse()
		if err is not None:
			raise err
			
		total = 1
		total, err = await db.get_rowcnt('datatable')
		if err is not None:
			raise err

		logger.debug('Total row count done! %s' % total)
		return total

	async def get_secrets(self):
		try:
			hexkey = False
			if isinstance(self.bootkey, str) and len(self.bootkey) == 32:
				try:
					bootkey = bytes.fromhex(self.bootkey)
					hexkey = True
				except:
					pass
			
			if hexkey is False:
				regfile = self.bootkey
				if isinstance(self.bootkey, str):
					regfile = AFile(self.bootkey)
				reghive = AIOWinRegHive(regfile)
				syshive = SYSTEM(reghive)
				bootkey = await syshive.get_bootkey()
			
			file = self.ntdsfile
			if isinstance(self.ntdsfile, str):
				file = AFile(self.ntdsfile)
			
			db = ESENT_DB(file)
			_, err = await db.parse()
			if err is not None:
				raise err
			
			ntds = NTDS(db, bootkey)			
			logger.debug('Dumping secrets')
			async for secret, err in ntds.dump_secrets(with_history=self.with_history, ignore_errors = self.ignore_errors):
				yield secret, err
			
		except Exception as e:
			yield None, e

async def flush_buffer(buffer, outfile_handle = None):
	try:
		if outfile_handle is not None:
			res = ''
			for secret in buffer:
				try:
					res += str(secret)
				except:
					continue
			outfile_handle.write(res)
		else:
			for secret in buffer:
				try:
					print(str(secret))
				except:
					continue
		
		buffer = []
		return True, None
	except Exception as e:
		return None, e


async def amain():
	import argparse
	import traceback
	try:
		parser = argparse.ArgumentParser(description='NTDS.dit parser')
		parser.add_argument('-v', '--verbose', action='count', default=0)
		parser.add_argument('-p', '--progress', action='store_true', help='Show progress bar')
		parser.add_argument('-o', '--out-file', help='Output file path.')
		parser.add_argument('--strict', action='store_true', help='Strict parsing. Fails on errors')
		parser.add_argument('--no-history', action='store_true', help='Do not parse history')
		parser.add_argument('bootkey',  help='Bootkey OR path to the SYSTEM hive file')
		parser.add_argument('ntdsfile', help='NTDS.dit file path')

		args = parser.parse_args()

		ntdscon = NTDSParserConsole(
			args.bootkey,
			args.ntdsfile,
			ignore_errors=args.strict,
			with_history=not args.no_history
		)

		buffer = []
		buffer_size = 1000
		total = await ntdscon.get_total_rows()
		if args.progress is True:
			pbar     = tqdm(desc='JET table parsing ', total=total, unit='records', miniters= total//200 ,position=0)
			pbar_sec = tqdm(desc='User secrets found', unit = '', miniters=buffer_size//10 ,position=1)

		outfile_handle = None
		if args.out_file is not None:
			outfile_handle = open(args.out_file, 'w', newline = '')

		async for secret, err in ntdscon.get_secrets():
			if err is not None:
				raise err

			if args.progress is True:
				pbar.update()
				
			if secret is None:
				continue
				
			if args.progress is True:
				pbar_sec.update()
				

			buffer.append(secret)
			if len(buffer) > buffer_size:
				_, err = await flush_buffer(buffer, outfile_handle)
				buffer = []
				if err is not None:
					raise err

			
		_, err = await flush_buffer(buffer, outfile_handle)
		buffer = []
		if err is not None:
			raise err
			
		
	except Exception as e:
		traceback.print_exc()


def main():
	asyncio.run(amain())

if __name__ == '__main__':
	main()
	


import enum

class SECRET_TYPE(enum.Enum):
	NTDS = 0
	NTDS_CLEARTEXT = 1
	NTDS_KERBEROS = 2

NAME_TO_INTERNAL = {
	'uSNCreated':b'ATTq131091',
	'uSNChanged':b'ATTq131192',
	'name':b'ATTm3',
	'objectGUID':b'ATTk589826',
	'objectSid':b'ATTr589970',
	'userAccountControl':b'ATTj589832',
	'primaryGroupID':b'ATTj589922',
	'accountExpires':b'ATTq589983',
	'logonCount':b'ATTj589993',
	'sAMAccountName':b'ATTm590045',
	'sAMAccountType':b'ATTj590126',
	'lastLogonTimestamp':b'ATTq589876',
	'userPrincipalName':b'ATTm590480',
	'unicodePwd':b'ATTk589914',
	'dBCSPwd':b'ATTk589879',
	'ntPwdHistory':b'ATTk589918',
	'lmPwdHistory':b'ATTk589984',
	'pekList':b'ATTk590689',
	'supplementalCredentials':b'ATTk589949',
	'pwdLastSet':b'ATTq589920',
}

NAME_TO_ATTRTYP = {
	'userPrincipalName': 0x90290,
	'sAMAccountName': 0x900DD,
	'unicodePwd': 0x9005A,
	'dBCSPwd': 0x90037,
	'ntPwdHistory': 0x9005E,
	'lmPwdHistory': 0x900A0,
	'supplementalCredentials': 0x9007D,
	'objectSid': 0x90092,
	'userAccountControl':0x90008,
}

ATTRTYP_TO_ATTID = {
	'userPrincipalName': '1.2.840.113556.1.4.656',
	'sAMAccountName': '1.2.840.113556.1.4.221',
	'unicodePwd': '1.2.840.113556.1.4.90',
	'dBCSPwd': '1.2.840.113556.1.4.55',
	'ntPwdHistory': '1.2.840.113556.1.4.94',
	'lmPwdHistory': '1.2.840.113556.1.4.160',
	'supplementalCredentials': '1.2.840.113556.1.4.125',
	'objectSid': '1.2.840.113556.1.4.146',
	'pwdLastSet': '1.2.840.113556.1.4.96',
	'userAccountControl':'1.2.840.113556.1.4.8',
}

KERBEROS_TYPE = {
	1:'dec-cbc-crc',
	3:'des-cbc-md5',
	17:'aes128-cts-hmac-sha1-96',
	18:'aes256-cts-hmac-sha1-96',
	0xffffff74:'rc4_hmac',
}

INTERNAL_TO_NAME = dict((v,k) for k,v in NAME_TO_INTERNAL.items())

SAM_NORMAL_USER_ACCOUNT = 0x30000000
SAM_MACHINE_ACCOUNT     = 0x30000001
SAM_TRUST_ACCOUNT       = 0x30000002

ACCOUNT_TYPES = ( SAM_NORMAL_USER_ACCOUNT, SAM_MACHINE_ACCOUNT, SAM_TRUST_ACCOUNT)


import io


class ESENT_PAGE_HEADER_2003_SP0:
	def __init__(self):
		self._len = 40
		self.CheckSum = None
		self.PageNumber = None
		self.LastModificationTime= None
		self.PreviousPageNumber= None
		self.NextPageNumber= None
		self.FatherDataPage= None
		self.AvailableDataSize= None
		self.AvailableUncommittedDataSize= None
		self.FirstAvailableDataOffset= None
		self.FirstAvailablePageTag= None
		self.PageFlags= None
	
	@staticmethod
	def from_bytes(data):
		return ESENT_PAGE_HEADER_2003_SP0.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_PAGE_HEADER_2003_SP0()
		res.CheckSum = int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.PageNumber = int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.LastModificationTime= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.PreviousPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.NextPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.FatherDataPage= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.AvailableDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.AvailableUncommittedDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailableDataOffset= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailablePageTag= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.PageFlags= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		return res

class ESENT_PAGE_HEADER_620_0b:
	def __init__(self):
		self._len = 44
		self.CheckSum = None
		self.ECCCheckSum = None
		self.LastModificationTime= None
		self.PreviousPageNumber= None
		self.NextPageNumber= None
		self.FatherDataPage= None
		self.AvailableDataSize= None
		self.AvailableUncommittedDataSize= None
		self.FirstAvailableDataOffset= None
		self.FirstAvailablePageTag= None
		self.PageFlags= None
	
	@staticmethod
	def from_bytes(data):
		return ESENT_PAGE_HEADER_620_0b.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_PAGE_HEADER_620_0b()
		res.CheckSum = int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.ECCCheckSum = int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.LastModificationTime= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.PreviousPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.NextPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.FatherDataPage= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.AvailableDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.AvailableUncommittedDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailableDataOffset= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailablePageTag= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.PageFlags= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		return res

class ESENT_PAGE_HEADER_WIN7:
	def __init__(self):
		self._len = 40
		self.CheckSum = None
		self.LastModificationTime= None
		self.PreviousPageNumber= None
		self.NextPageNumber= None
		self.FatherDataPage= None
		self.AvailableDataSize= None
		self.AvailableUncommittedDataSize= None
		self.FirstAvailableDataOffset= None
		self.FirstAvailablePageTag= None
		self.PageFlags= None
	
	@staticmethod
	def from_bytes(data):
		return ESENT_PAGE_HEADER_WIN7.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_PAGE_HEADER_WIN7()
		res.CheckSum = int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.LastModificationTime= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.PreviousPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.NextPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.FatherDataPage= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.AvailableDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.AvailableUncommittedDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailableDataOffset= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailablePageTag= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.PageFlags= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		return res

class ESENT_PAGE_HEADER_WIN7_EXT:
	def __init__(self):
		self._len = 80
		self.CheckSum = None
		self.LastModificationTime= None
		self.PreviousPageNumber= None
		self.NextPageNumber= None
		self.FatherDataPage= None
		self.AvailableDataSize= None
		self.AvailableUncommittedDataSize= None
		self.FirstAvailableDataOffset= None
		self.FirstAvailablePageTag= None
		self.PageFlags= None
		self.ExtendedCheckSum1 = None
		self.ExtendedCheckSum2 = None
		self.ExtendedCheckSum3 = None
		self.PageNumber = None
		self.Unknown = None
	
	@staticmethod
	def from_bytes(data):
		return ESENT_PAGE_HEADER_WIN7_EXT.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_PAGE_HEADER_WIN7_EXT()
		res.CheckSum = int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.LastModificationTime= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.PreviousPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.NextPageNumber= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.FatherDataPage= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.AvailableDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.AvailableUncommittedDataSize= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailableDataOffset= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.FirstAvailablePageTag= int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.PageFlags= int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.ExtendedCheckSum1= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.ExtendedCheckSum2= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.ExtendedCheckSum3= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.PageNumber= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		res.Unknown= int.from_bytes(buff.read(8), byteorder='little', signed=False)
		return res


async def readheader(version, revision, pagesize=8192, data=None):
	try:
		if (version < 0x620) or (version == 0x620 and revision < 0x0b):
			return ESENT_PAGE_HEADER_2003_SP0.from_bytes(data), None
		elif version == 0x620 and revision < 0x11:
			# Exchange 2003 SP1 and Windows Vista and later
			return ESENT_PAGE_HEADER_620_0b.from_bytes(data), None
		else:
			if pagesize > 8192:
				return ESENT_PAGE_HEADER_WIN7_EXT.from_bytes(data), None
			return ESENT_PAGE_HEADER_WIN7.from_bytes(data), None
	except Exception as e:
		return None, e


import io

class ESENT_LEAF_ENTRY_COMMON:
	def __init__(self):
		self.CommonPageKeySize = None
		self.LocalPageKeySize = None
		self.LocalPageKey = None #  ('LocalPageKey',':'),
		self.EntryData = None # ('EntryData',':'),
	
	@staticmethod
	def from_bytes(data):
		return ESENT_LEAF_ENTRY_COMMON.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_LEAF_ENTRY_COMMON()
		res.CommonPageKeySize = int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.LocalPageKeySize = int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.LocalPageKey = buff.read(res.LocalPageKeySize)
		res.EntryData = buff.read()
		return res

class ESENT_LEAF_ENTRY:
	def __init__(self):
		self.LocalPageKeySize = None
		self.LocalPageKey = None #  ('LocalPageKey',':'),
		self.EntryData = None # ('EntryData',':'),
	
	@staticmethod
	def from_bytes(data):
		return ESENT_LEAF_ENTRY.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_LEAF_ENTRY()
		res.LocalPageKeySize = int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.LocalPageKey = buff.read(res.LocalPageKeySize)
		res.EntryData = buff.read()
		return res

async def readleafentry(flags, data):
	try:
		if flags & 0x4 > 0:
			return ESENT_LEAF_ENTRY_COMMON.from_bytes(data), None
		return ESENT_LEAF_ENTRY.from_bytes(data), None

	except Exception as e:
		return None, e


import io

class ESENT_SPACE_TREE_ENTRY:
	def __init__(self):
		self.PageKeySize = None
		self.LastPageNumber = None
		self.NumberOfPages = None
	
	@staticmethod
	def from_bytes(data):
		return ESENT_SPACE_TREE_ENTRY.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = ESENT_SPACE_TREE_ENTRY()
		res.PageKeySize = int.from_bytes(buff.read(2), byteorder='little', signed=False)
		res.LastPageNumber = int.from_bytes(buff.read(4), byteorder='little', signed=False)
		res.NumberOfPages = int.from_bytes(buff.read(4), byteorder='little', signed=False)
		return res



import itertools
class UserSecrets:
	def __init__(self):
		self.domain = None
		self.username = None
		self.nt_hash = None
		self.lm_hash = None
		
		self.object_sid = None
		self.pwd_last_set = None
		self.user_account_status = None
		self.user_account_control = None
		
		self.lm_history = []
		self.nt_history = []
		self.kerberos_keys = []
		self.cleartext_pwds = []
	
	def to_dict(self):
		t = {}
		t['domain'] = self.domain
		t['username'] = self.username
		t['nt_hash'] = self.nt_hash.hex()
		t['lm_hash'] = self.lm_hash.hex()

		t['object_sid'] = str(self.object_sid)
		t['pwd_last_set'] = str(self.pwd_last_set)
		t['user_account_control'] = str(self.user_account_control)
		
		t['lm_history'] = []
		for i, lm in enumerate(self.lm_history):
			t['lm_history'].append(lm.hex())
		
		t['nt_history'] = []
		for i, nt in enumerate(self.nt_history):
			t['nt_history'].append(nt.hex())
		
		t['cleartext_pwds'] = []
		for i, pwd in enumerate(self.cleartext_pwds):
			t['cleartext_pwds'].append(str(pwd))
			
		t['kerberos_keys'] = []
		for ktype, key in self.kerberos_keys:
			t['kerberos_keys'].append([str(ktype), key.hex()])

		
		return t
	def __str__(self):
		t = ''
		t += ':'.join(['ntlm', str(self.domain),str(self.username),str(self.user_account_control),str(self.object_sid),self.lm_hash.hex(), self.nt_hash.hex(),str(self.pwd_last_set)])
		t += '\r\n'
		for i, x in enumerate(itertools.zip_longest(self.lm_history[1:],self.nt_history[1:])):
			lm, nt = x
			if lm is None:
				lm = bytes.fromhex('aad3b435b51404eeaad3b435b51404ee')
			if nt is None:
				nt = bytes.fromhex('31d6cfe0d16ae931b73c59d7e0c089c0')
			t += ':'.join(['ntlm_history', str(self.domain),str(self.username),str(self.user_account_control),str(self.object_sid), lm.hex(), nt.hex(), 'history_%d'% (i) ])
			t += '\r\n'
		for ktype, key in self.kerberos_keys:
			t += ':'.join(['kerberos',str(self.domain),str(self.username),str(self.object_sid),ktype,key.hex()])
			t += '\r\n'
		for key in self.cleartext_pwds:
			t += ':'.join(['cleartext',str(self.domain),str(self.username),str(self.object_sid),key])
			t += '\r\n'
			
		return t


import io

class SAMR_RPC_SID_IDENTIFIER_AUTHORITY:
	def __init__(self):
		self.Value = None
	
	@staticmethod
	def from_bytes(data):
		return SAMR_RPC_SID_IDENTIFIER_AUTHORITY.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = SAMR_RPC_SID_IDENTIFIER_AUTHORITY()
		res.Value = buff.read(6)
		return res

class SAMR_RPC_SID:
	def __init__(self):
		self.Revision = None
		self.SubAuthorityCount = None
		self.IdentifierAuthority  = None
		self.SubLen  = None
		self.SubAuthority = None
	
	@staticmethod
	def from_bytes(data):
		return SAMR_RPC_SID.from_buffer(io.BytesIO(data))
	
	@staticmethod
	def from_buffer(buff):
		res = SAMR_RPC_SID()
		res.Revision = buff.read(1)[0]
		res.SubAuthorityCount = buff.read(1)[0]
		res.IdentifierAuthority  = SAMR_RPC_SID_IDENTIFIER_AUTHORITY.from_buffer(buff)
		res.SubLen  = res.SubAuthorityCount*4
		res.SubAuthority = buff.read()
		return res

	def __str__(self):
	   ans = 'S-%d-%d' % (self.Revision, ord(self.IdentifierAuthority.Value[5:6]))
	   for i in range(self.SubAuthorityCount):
		   t = int.from_bytes(self.SubAuthority[i*4:i*4+4], byteorder='big', signed = False)
		   ans += '-%d' % t
	   return ans


ATTR_TO_COLNAME = {
	b"ATTq589983" : "ACCOUNT_EXPIRES",
	b"ATTm591131" : "ACCOUNT_NAME_HISTORY",
	b"ATTq590584" : "ACS_AGGREGATE_TOKEN_RATE_PER_USER",
	b"ATTq590590" : "ACS_ALLOCABLE_RSVP_BANDWIDTH",
	b"ATTj590603" : "ACS_CACHE_TIMEOUT",
	b"ATTj590581" : "ACS_DIRECTION",
	b"ATTj590602" : "ACS_DSBM_DEADTIME",
	b"ATTj590600" : "ACS_DSBM_PRIORITY",
	b"ATTj590601" : "ACS_DSBM_REFRESH",
	b"ATTi590594" : "ACS_ENABLE_ACS_SERVICE",
	b"ATTi590723" : "ACS_ENABLE_RSVP_ACCOUNTING",
	b"ATTi590592" : "ACS_ENABLE_RSVP_MESSAGE_LOGGING",
	b"ATTj590593" : "ACS_EVENT_LOG_LEVEL",
	b"ATTm590608" : "ACS_IDENTITY_NAME",
	b"ATTq590721" : "ACS_MAX_AGGREGATE_PEAK_RATE_PER_USER",
	b"ATTj590585" : "ACS_MAX_DURATION_PER_FLOW",
	b"ATTj590725" : "ACS_MAX_NO_OF_ACCOUNT_FILES",
	b"ATTj590598" : "ACS_MAX_NO_OF_LOG_FILES",
	b"ATTq590591" : "ACS_MAX_PEAK_BANDWIDTH",
	b"ATTq590583" : "ACS_MAX_PEAK_BANDWIDTH_PER_FLOW",
	b"ATTj590726" : "ACS_MAX_SIZE_OF_RSVP_ACCOUNT_FILE",
	b"ATTj590599" : "ACS_MAX_SIZE_OF_RSVP_LOG_FILE",
	b"ATTq591137" : "ACS_MAX_TOKEN_BUCKET_PER_FLOW",
	b"ATTq590582" : "ACS_MAX_TOKEN_RATE_PER_FLOW",
	b"ATTq591138" : "ACS_MAXIMUM_SDU_SIZE",
	b"ATTq591141" : "ACS_MINIMUM_DELAY_VARIATION",
	b"ATTq591140" : "ACS_MINIMUM_LATENCY",
	b"ATTq591139" : "ACS_MINIMUM_POLICED_SIZE",
	b"ATTq591144" : "ACS_NON_RESERVED_MAX_SDU_SIZE",
	b"ATTq591145" : "ACS_NON_RESERVED_MIN_POLICED_SIZE",
	b"ATTq591142" : "ACS_NON_RESERVED_PEAK_RATE",
	b"ATTq591143" : "ACS_NON_RESERVED_TOKEN_SIZE",
	b"ATTq590604" : "ACS_NON_RESERVED_TX_LIMIT",
	b"ATTq590722" : "ACS_NON_RESERVED_TX_SIZE",
	b"ATTq590589" : "ACS_PERMISSION_BITS",
	b"ATTm590596" : "ACS_POLICY_NAME",
	b"ATTj590588" : "ACS_PRIORITY",
	b"ATTm590724" : "ACS_RSVP_ACCOUNT_FILES_LOCATION",
	b"ATTm590597" : "ACS_RSVP_LOG_FILES_LOCATION",
	b"ATTj590586" : "ACS_SERVICE_TYPE",
	b"ATTm590580" : "ACS_TIME_OF_DAY",
	b"ATTj590587" : "ACS_TOTAL_NO_OF_FLOWS",
	b"ATTm591136" : "ACS_SERVER_LIST",
	b"ATTm590089" : "ADDITIONAL_INFORMATION",
	b"ATTm590713" : "ADDITIONAL_TRUSTED_SERVICE_NAMES",
	b"ATTm131328" : "ADDRESS",
	b"ATTb591068" : "ADDRESS_BOOK_ROOTS",
	b"ATTk131396" : "ADDRESS_ENTRY_DISPLAY_TABLE",
	b"ATTk131472" : "ADDRESS_ENTRY_DISPLAY_TABLE_MSDOS",
	b"ATTm131689" : "ADDRESS_HOME",
	b"ATTk131327" : "ADDRESS_SYNTAX",
	b"ATTe131422" : "ADDRESS_TYPE",
	b"ATTm590438" : "ADMIN_CONTEXT_MENU",
	b"ATTj589974" : "ADMIN_COUNT",
	b"ATTm131298" : "ADMIN_DESCRIPTION",
	b"ATTm131266" : "ADMIN_DISPLAY_NAME",
	b"ATTm591514" : "ADMIN_MULTISELECT_PROPERTY_PAGES",
	b"ATTm590386" : "ADMIN_PROPERTY_PAGES",
	b"ATTc590737" : "ALLOWED_ATTRIBUTES",
	b"ATTc590738" : "ALLOWED_ATTRIBUTES_EFFECTIVE",
	b"ATTc590735" : "ALLOWED_CHILD_CLASSES",
	b"ATTc590736" : "ALLOWED_CHILD_CLASSES_EFFECTIVE",
	b"ATTm590691" : "ALT_SECURITY_IDENTITIES",
	b"ATTm591032" : "ANR",
	b"ATTj590672" : "APP_SCHEMA_VERSION",
	b"ATTm590042" : "APPLICATION_NAME",
	b"ATTm590165" : "APPLIES_TO",
	b"ATTm590107" : "ASSET_NUMBER",
	b"ATTb590476" : "ASSISTANT",
	b"ATTk591037" : "ASSOC_NT_ACCOUNT",
	b"ATTf1376293" : "ASSOCIATEDDOMAIN",
	b"ATTb1376294" : "ASSOCIATEDNAME",
	b"ATTk58" : "ATTRIBUTECERTIFICATEATTRIBUTE",
	b"ATTm590572" : "ATTRIBUTE_DISPLAY_NAMES",
	b"ATTc131102" : "ATTRIBUTE_ID",
	b"ATTk589973" : "ATTRIBUTE_SECURITY_GUID",
	b"ATTc131104" : "ATTRIBUTE_SYNTAX",
	b"ATTm1572869" : "ATTRIBUTE_TYPES",
	b"ATTk1376311" : "AUDIO",
	b"ATTk590026" : "AUDITING_POLICY",
	b"ATTj589835" : "AUTHENTICATION_OPTIONS",
	b"ATTk38" : "AUTHORITY_REVOCATION_LIST",
	b"ATTc131423" : "AUXILIARY_CLASS",
	b"ATTq589873" : "BAD_PASSWORD_TIME",
	b"ATTj589836" : "BAD_PWD_COUNT",
	b"ATTk590156" : "BIRTH_LOCATION",
	b"ATTb590644" : "BRIDGEHEAD_SERVER_LIST_BL",
	b"ATTb590643" : "BRIDGEHEAD_TRANSPORT_LIST",
	b"ATTm1376304" : "BUILDINGNAME",
	b"ATTq589837" : "BUILTIN_CREATION_TIME",
	b"ATTq589838" : "BUILTIN_MODIFIED_COUNT",
	b"ATTm15" : "BUSINESS_CATEGORY",
	b"ATTj590108" : "BYTES_PER_MINUTE",
	b"ATTk37" : "CA_CERTIFICATE",
	b"ATTm590521" : "CA_CERTIFICATE_DN",
	b"ATTm590511" : "CA_CONNECT",
	b"ATTm590514" : "CA_USAGES",
	b"ATTm590512" : "CA_WEB_URL",
	b"ATTm590639" : "CAN_UPGRADE_SCRIPT",
	b"ATTm590740" : "CANONICAL_NAME",
	b"ATTm1441793" : "CARLICENSE",
	b"ATTm590499" : "CATALOGS",
	b"ATTm590496" : "CATEGORIES",
	b"ATTk590146" : "CATEGORY_ID",
	b"ATTb590508" : "CERTIFICATE_AUTHORITY_OBJECT",
	b"ATTk39" : "CERTIFICATE_REVOCATION_LIST",
	b"ATTm590647" : "CERTIFICATE_TEMPLATES",
	b"ATTm590434" : "CLASS_DISPLAY_NAME",
	b"ATTj589840" : "CODE_PAGE",
	b"ATTm589843" : "COM_CLASSID",
	b"ATTm590073" : "COM_CLSID",
	b"ATTm589844" : "COM_INTERFACEID",
	b"ATTm590077" : "COM_OTHER_PROG_ID",
	b"ATTm589845" : "COM_PROGID",
	b"ATTm590075" : "COM_TREAT_AS_CLASS_ID",
	b"ATTm590078" : "COM_TYPELIB_ID",
	b"ATTm590074" : "COM_UNIQUE_LIBID",
	b"ATTm131153" : "COMMENT",
	b"ATTm3" : "COMMON_NAME",
	b"ATTm131218" : "COMPANY",
	b"ATTi589848" : "CONTENT_INDEXING_ALLOWED",
	b"ATTm590323" : "CONTEXT_MENU",
	b"ATTk590024" : "CONTROL_ACCESS_RIGHTS",
	b"ATTj131207" : "COST",
	b"ATTj589849" : "COUNTRY_CODE",
	b"ATTm6" : "COUNTRY_NAME",
	b"ATTm590634" : "CREATE_DIALOG",
	b"ATTl1638401" : "CREATE_TIME_STAMP",
	b"ATTm590636" : "CREATE_WIZARD_EXT",
	b"ATTq589850" : "CREATION_TIME",
	b"ATTm590322" : "CREATION_WIZARD",
	b"ATTm590503" : "CREATOR",
	b"ATTb590513" : "CRL_OBJECT",
	b"ATTk590507" : "CRL_PARTITIONED_REVOCATION_LIST",
	b"ATTk40" : "CROSS_CERTIFICATE_PAIR",
	b"ATTk590161" : "CURR_MACHINE_ID",
	b"ATTk590159" : "CURRENT_LOCATION",
	b"ATTb590520" : "CURRENT_PARENT_CA",
	b"ATTk589851" : "CURRENT_VALUE",
	b"ATTk589879" : "DBCS_PWD",
	b"ATTb590037" : "DEFAULT_CLASS_STORE",
	b"ATTb590304" : "DEFAULT_GROUP",
	b"ATTi590342" : "DEFAULT_HIDING_VALUE",
	b"ATTb589881" : "DEFAULT_LOCAL_POLICY_OBJECT",
	b"ATTb590607" : "DEFAULT_OBJECT_CATEGORY",
	b"ATTj590056" : "DEFAULT_PRIORITY",
	b"ATTm590048" : "DEFAULT_SECURITY_DESCRIPTOR",
	b"ATTk53" : "DELTA_REVOCATION_LIST",
	b"ATTm131213" : "DEPARTMENT",
	b"ATTm1441794" : "DEPARTMENTNUMBER",
	b"ATTm13" : "DESCRIPTION",
	b"ATTm590170" : "DESKTOP_PROFILE",
	b"ATTf27" : "DESTINATION_INDICATOR",
	b"ATTk590539" : "DHCP_CLASSES",
	b"ATTq590524" : "DHCP_FLAGS",
	b"ATTm590525" : "DHCP_IDENTIFICATION",
	b"ATTf590530" : "DHCP_MASK",
	b"ATTq590543" : "DHCP_MAXKEY",
	b"ATTm590527" : "DHCP_OBJ_DESCRIPTION",
	b"ATTm590526" : "DHCP_OBJ_NAME",
	b"ATTk590538" : "DHCP_OPTIONS",
	b"ATTk590542" : "DHCP_PROPERTIES",
	b"ATTf590531" : "DHCP_RANGES",
	b"ATTf590533" : "DHCP_RESERVATIONS",
	b"ATTf590528" : "DHCP_SERVERS",
	b"ATTf590532" : "DHCP_SITES",
	b"ATTf590541" : "DHCP_STATE",
	b"ATTf590529" : "DHCP_SUBNETS",
	b"ATTj590523" : "DHCP_TYPE",
	b"ATTq590522" : "DHCP_UNIQUE_KEY",
	b"ATTq590544" : "DHCP_UPDATE_TIME",
	b"ATTm131085" : "DISPLAY_NAME",
	b"ATTf131425" : "DISPLAY_NAME_PRINTABLE",
	b"ATTm1572866" : "DIT_CONTENT_RULES",
	b"ATTm590085" : "DIVISION",
	b"ATTb131108" : "DMD_LOCATION",
	b"ATTm131670" : "DMD_NAME",
	b"ATTb591066" : "DN_REFERENCE_UPDATE",
	b"ATTi590202" : "DNS_ALLOW_DYNAMIC",
	b"ATTi590203" : "DNS_ALLOW_XFR",
	b"ATTm590443" : "DNS_HOST_NAME",
	b"ATTj590205" : "DNS_NOTIFY_SECONDARIES",
	b"ATTk591130" : "DNS_PROPERTY",
	b"ATTk590206" : "DNS_RECORD",
	b"ATTm589852" : "DNS_ROOT",
	b"ATTj590204" : "DNS_SECURE_SECONDARIES",
	b"ATTi591238" : "DNS_TOMBSTONED",
	b"ATTb1376270" : "DOCUMENTAUTHOR",
	b"ATTm1376267" : "DOCUMENTIDENTIFIER",
	b"ATTm1376271" : "DOCUMENTLOCATION",
	b"ATTm1376312" : "DOCUMENTPUBLISHER",
	b"ATTm1376268" : "DOCUMENTTITLE",
	b"ATTm1376269" : "DOCUMENTVERSION",
	b"ATTb590492" : "DOMAIN_CERTIFICATE_AUTHORITIES",
	b"ATTm1376281" : "DOMAIN_COMPONENT",
	b"ATTb590296" : "DOMAIN_CROSS_REF",
	b"ATTb590510" : "DOMAIN_ID",
	b"ATTj590579" : "DOMAIN_IDENTIFIER",
	b"ATTb589856" : "DOMAIN_POLICY_OBJECT",
	b"ATTb590246" : "DOMAIN_POLICY_REFERENCE",
	b"ATTm589982" : "DOMAIN_REPLICA",
	b"ATTk590245" : "DOMAIN_WIDE_POLICY",
	b"ATTm1376261" : "DRINK",
	b"ATTm590053" : "DRIVER_NAME",
	b"ATTj590100" : "DRIVER_VERSION",
	b"ATTl591181" : "DS_CORE_PROPAGATION_DATA",
	b"ATTm131284" : "DS_HEURISTICS",
	b"ATTj591168" : "DS_UI_ADMIN_MAXIMUM",
	b"ATTm591167" : "DS_UI_ADMIN_NOTIFICATION",
	b"ATTj591169" : "DS_UI_SHELL_MAXIMUM",
	b"ATTk131146" : "DSA_SIGNATURE",
	b"ATTb590361" : "DYNAMIC_LDAP_SERVER",
	b"ATTm1376259" : "E_MAIL_ADDRESSES",
	b"ATTk590092" : "EFSPOLICY",
	b"ATTm589859" : "EMPLOYEE_ID",
	b"ATTm131682" : "EMPLOYEE_NUMBER",
	b"ATTm131685" : "EMPLOYEE_TYPE",
	b"ATTi131629" : "ENABLED",
	b"ATTi589860" : "ENABLED_CONNECTION",
	b"ATTm590649" : "ENROLLMENT_PROVIDERS",
	b"ATTm590733" : "EXTENDED_ATTRIBUTE_INFO",
	b"ATTi131452" : "EXTENDED_CHARS_ALLOWED",
	b"ATTm590732" : "EXTENDED_CLASS_INFO",
	b"ATTm131299" : "EXTENSION_NAME",
	b"ATTm591511" : "EXTRA_COLUMNS",
	b"ATTm23" : "FACSIMILE_TELEPHONE_NUMBER",
	b"ATTm590640" : "FILE_EXT_PRIORITY",
	b"ATTj589862" : "FLAGS",
	b"ATTm590335" : "FLAT_NAME",
	b"ATTq589863" : "FORCE_LOGOFF",
	b"ATTk590180" : "FOREIGN_IDENTIFIER",
	b"ATTm590506" : "FRIENDLY_NAMES",
	b"ATTi590734" : "FROM_ENTRY",
	b"ATTb589864" : "FROM_SERVER",
	b"ATTb590693" : "FRS_COMPUTER_REFERENCE",
	b"ATTb590694" : "FRS_COMPUTER_REFERENCE_BL",
	b"ATTm590695" : "FRS_CONTROL_DATA_CREATION",
	b"ATTm590696" : "FRS_CONTROL_INBOUND_BACKLOG",
	b"ATTm590697" : "FRS_CONTROL_OUTBOUND_BACKLOG",
	b"ATTm590308" : "FRS_DIRECTORY_FILTER",
	b"ATTj590314" : "FRS_DS_POLL",
	b"ATTk590360" : "FRS_EXTENSIONS",
	b"ATTm590315" : "FRS_FAULT_CONDITION",
	b"ATTm590307" : "FRS_FILE_FILTER",
	b"ATTj590698" : "FRS_FLAGS",
	b"ATTj590358" : "FRS_LEVEL_LIMIT",
	b"ATTb590699" : "FRS_MEMBER_REFERENCE",
	b"ATTb590700" : "FRS_MEMBER_REFERENCE_BL",
	b"ATTj590701" : "FRS_PARTNER_AUTH_LEVEL",
	b"ATTb590702" : "FRS_PRIMARY_MEMBER",
	b"ATTk590357" : "FRS_REPLICA_SET_GUID",
	b"ATTj589855" : "FRS_REPLICA_SET_TYPE",
	b"ATTm590311" : "FRS_ROOT_PATH",
	b"ATTp590359" : "FRS_ROOT_SECURITY",
	b"ATTm590324" : "FRS_SERVICE_COMMAND",
	b"ATTm590703" : "FRS_SERVICE_COMMAND_STATUS",
	b"ATTm590312" : "FRS_STAGING_PATH",
	b"ATTl590704" : "FRS_TIME_LAST_COMMAND",
	b"ATTl590705" : "FRS_TIME_LAST_CONFIG_CHANGE",
	b"ATTj590309" : "FRS_UPDATE_TIMEOUT",
	b"ATTm590706" : "FRS_VERSION",
	b"ATTk589867" : "FRS_VERSION_GUID",
	b"ATTm590310" : "FRS_WORKING_PATH",
	b"ATTb590193" : "FSMO_ROLE_OWNER",
	b"ATTj131373" : "GARBAGE_COLL_PERIOD",
	b"ATTi589865" : "GENERATED_CONNECTION",
	b"ATTm44" : "GENERATION_QUALIFIER",
	b"ATTm42" : "GIVEN_NAME",
	b"ATTb591069" : "GLOBAL_ADDRESS_LIST",
	b"ATTc131094" : "GOVERNS_ID",
	b"ATTm590715" : "GP_LINK",
	b"ATTj590716" : "GP_OPTIONS",
	b"ATTm590718" : "GPC_FILE_SYS_PATH",
	b"ATTj590717" : "GPC_FUNCTIONALITY_VERSION",
	b"ATTm591172" : "GPC_MACHINE_EXTENSION_NAMES",
	b"ATTm591173" : "GPC_USER_EXTENSION_NAMES",
	b"ATTm591518" : "GPC_WQL_FILTER",
	b"ATTj589976" : "GROUP_ATTRIBUTES",
	b"ATTk589990" : "GROUP_MEMBERSHIP_SAM",
	b"ATTm590169" : "GROUP_PRIORITY",
	b"ATTj590574" : "GROUP_TYPE",
	b"ATTm590168" : "GROUPS_TO_IGNORE",
	b"ATTb131086" : "HAS_MASTER_NCS",
	b"ATTb131087" : "HAS_PARTIAL_REPLICA_NCS",
	b"ATTk131474" : "HELP_DATA16",
	b"ATTk131081" : "HELP_DATA32",
	b"ATTm131399" : "HELP_FILE_NAME",
	b"ATTi591604" : "HIDE_FROM_AB",
	b"ATTm589868" : "HOME_DIRECTORY",
	b"ATTm589869" : "HOME_DRIVE",
	b"ATTm51" : "HOUSEIDENTIFIER",
	b"ATTm1376265" : "HOST",
	b"ATTm590043" : "ICON_PATH",
	b"ATTk590144" : "IMPLEMENTED_CATEGORIES",
	b"ATTm590505" : "INDEXEDSCOPES",
	b"ATTm590363" : "INITIAL_AUTH_INCOMING",
	b"ATTm590364" : "INITIAL_AUTH_OUTGOING",
	b"ATTm43" : "INITIALS",
	b"ATTj590671" : "INSTALL_UI_LEVEL",
	b"ATTj131073" : "INSTANCE_TYPE",
	b"ATTj591072" : "INTER_SITE_TOPOLOGY_FAILOVER",
	b"ATTb591070" : "INTER_SITE_TOPOLOGY_GENERATOR",
	b"ATTj591071" : "INTER_SITE_TOPOLOGY_RENEW",
	b"ATTg25" : "INTERNATIONAL_ISDN_NUMBER",
	b"ATTk131187" : "INVOCATION_ID",
	b"ATTk590447" : "IPSEC_DATA",
	b"ATTj590446" : "IPSEC_DATA_TYPE",
	b"ATTb590453" : "IPSEC_FILTER_REFERENCE",
	b"ATTm590445" : "IPSEC_ID",
	b"ATTb590450" : "IPSEC_ISAKMP_REFERENCE",
	b"ATTm590444" : "IPSEC_NAME",
	b"ATTm590712" : "IPSEC_NEGOTIATION_POLICY_ACTION",
	b"ATTb590452" : "IPSEC_NEGOTIATION_POLICY_REFERENCE",
	b"ATTm590711" : "IPSEC_NEGOTIATION_POLICY_TYPE",
	b"ATTb590451" : "IPSEC_NFA_REFERENCE",
	b"ATTb590448" : "IPSEC_OWNERS_REFERENCE",
	b"ATTb590341" : "IPSEC_POLICY_REFERENCE",
	b"ATTi590692" : "IS_CRITICAL_SYSTEM_OBJECT",
	b"ATTi590485" : "IS_DEFUNCT",
	b"ATTi131120" : "IS_DELETED",
	b"ATTi591036" : "IS_EPHEMERAL",
	b"ATTb131174" : "IS_MEMBER_OF_DL",
	b"ATTi590463" : "IS_MEMBER_OF_PARTIAL_ATTRIBUTE_SET",
	b"ATTb590462" : "IS_PRIVILEGE_HOLDER",
	b"ATTi131105" : "IS_SINGLE_VALUED",
	b"ATTk1376316" : "JPEGPHOTO",
	b"ATTm589872" : "KEYWORDS",
	b"ATTe2" : "KNOWLEDGE_INFORMATION",
	b"ATTq590343" : "LAST_BACKUP_RESTORATION_TIME",
	b"ATTq589874" : "LAST_CONTENT_INDEXED",
	b"ATTb590605" : "LAST_KNOWN_PARENT",
	b"ATTq589875" : "LAST_LOGOFF",
	b"ATTq589876" : "LAST_LOGON",
	b"ATTq591520" : "LAST_LOGON_TIMESTAMP",
	b"ATTq589877" : "LAST_SET_TIME",
	b"ATTm590154" : "LAST_UPDATE_SEQUENCE",
	b"ATTm590667" : "LDAP_ADMIN_LIMITS",
	b"ATTm131532" : "LDAP_DISPLAY_NAME",
	b"ATTk590668" : "LDAP_IPDENY_LIST",
	b"ATTe590479" : "LEGACY_EXCHANGE_DN",
	b"ATTj131122" : "LINK_ID",
	b"ATTk590093" : "LINK_TRACK_SECRET",
	b"ATTk589984" : "LM_PWD_HISTORY",
	b"ATTj589880" : "LOCAL_POLICY_FLAGS",
	b"ATTb590281" : "LOCAL_POLICY_REFERENCE",
	b"ATTj589882" : "LOCALE_ID",
	b"ATTm7" : "LOCALITY_NAME",
	b"ATTm590641" : "LOCALIZED_DESCRIPTION",
	b"ATTj591177" : "LOCALIZATION_DISPLAY_ID",
	b"ATTm590046" : "LOCATION",
	b"ATTq589885" : "LOCK_OUT_OBSERVATION_WINDOW",
	b"ATTq589884" : "LOCKOUT_DURATION",
	b"ATTj589897" : "LOCKOUT_THRESHOLD",
	b"ATTq590486" : "LOCKOUT_TIME",
	b"ATTk1441828" : "LOGO",
	b"ATTj589993" : "LOGON_COUNT",
	b"ATTk589888" : "LOGON_HOURS",
	b"ATTk589889" : "LOGON_WORKSTATION",
	b"ATTq589890" : "LSA_CREATION_TIME",
	b"ATTq589891" : "LSA_MODIFIED_COUNT",
	b"ATTj589892" : "MACHINE_ARCHITECTURE",
	b"ATTq590344" : "MACHINE_PASSWORD_CHANGE_INTERVAL",
	b"ATTj589895" : "MACHINE_ROLE",
	b"ATTk590283" : "MACHINE_WIDE_POLICY",
	b"ATTb590477" : "MANAGED_BY",
	b"ATTb590478" : "MANAGED_OBJECTS",
	b"ATTb1376266" : "MANAGER",
	b"ATTj131121" : "MAPI_ID",
	b"ATTk589896" : "MARSHALLED_INTERFACE",
	b"ATTb591233" : "MASTERED_BY",
	b"ATTq589898" : "MAX_PWD_AGE",
	b"ATTq589899" : "MAX_RENEW_AGE",
	b"ATTq589900" : "MAX_STORAGE",
	b"ATTq589901" : "MAX_TICKET_AGE",
	b"ATTc131097" : "MAY_CONTAIN",
	b"ATTm590406" : "MEETINGADVERTISESCOPE",
	b"ATTm590397" : "MEETINGAPPLICATION",
	b"ATTj590413" : "MEETINGBANDWIDTH",
	b"ATTk590414" : "MEETINGBLOB",
	b"ATTm590402" : "MEETINGCONTACTINFO",
	b"ATTm590391" : "MEETINGDESCRIPTION",
	b"ATTl590412" : "MEETINGENDTIME",
	b"ATTm590389" : "MEETINGID",
	b"ATTm590404" : "MEETINGIP",
	b"ATTm590409" : "MEETINGISENCRYPTED",
	b"ATTm590392" : "MEETINGKEYWORD",
	b"ATTm590398" : "MEETINGLANGUAGE",
	b"ATTm590393" : "MEETINGLOCATION",
	b"ATTj590400" : "MEETINGMAXPARTICIPANTS",
	b"ATTm590390" : "MEETINGNAME",
	b"ATTm590401" : "MEETINGORIGINATOR",
	b"ATTm590403" : "MEETINGOWNER",
	b"ATTm590394" : "MEETINGPROTOCOL",
	b"ATTm590408" : "MEETINGRATING",
	b"ATTm590410" : "MEETINGRECURRENCE",
	b"ATTm590405" : "MEETINGSCOPE",
	b"ATTl590411" : "MEETINGSTARTTIME",
	b"ATTm590395" : "MEETINGTYPE",
	b"ATTm590407" : "MEETINGURL",
	b"ATTb31" : "MEMBER",
	b"ATTm590474" : "MHS_OR_ADDRESS",
	b"ATTq589902" : "MIN_PWD_AGE",
	b"ATTj589903" : "MIN_PWD_LENGTH",
	b"ATTq589904" : "MIN_TICKET_AGE",
	b"ATTq589992" : "MODIFIED_COUNT",
	b"ATTq589905" : "MODIFIED_COUNT_AT_LAST_PROM",
	b"ATTl1638402" : "MODIFY_TIME_STAMP",
	b"ATTk589906" : "MONIKER",
	b"ATTm589907" : "MONIKER_DISPLAY_NAME",
	b"ATTk591129" : "MOVE_TREE_STATE",
	b"ATTb591251" : "MS_COM_DEFAULTPARTITIONLINK",
	b"ATTk591252" : "MS_COM_OBJECTID",
	b"ATTb591247" : "MS_COM_PARTITIONLINK",
	b"ATTb591248" : "MS_COM_PARTITIONSETLINK",
	b"ATTb591249" : "MS_COM_USERLINK",
	b"ATTb591250" : "MS_COM_USERPARTITIONSETLINK",
	b"ATTk591667" : "MS_DRM_IDENTITY_CERTIFICATE",
	b"ATTm591541" : "MS_DS_ADDITIONAL_DNS_HOST_NAME",
	b"ATTm591542" : "MS_DS_ADDITIONAL_SAM_ACCOUNT_NAME",
	b"ATTj591613" : "MS_DS_ALL_USERS_TRUST_QUOTA",
	b"ATTm591534" : "MS_DS_ALLOWED_DNS_SUFFIXES",
	b"ATTm591611" : "MS_DS_ALLOWED_TO_DELEGATE_TO",
	b"ATTc591282" : "MS_DS_AUXILIARY_CLASSES",
	b"ATTj591493" : "MS_DS_APPROX_IMMED_SUBORDINATES",
	b"ATTm591643" : "MS_DS_AZ_APPLICATION_DATA",
	b"ATTm591622" : "MS_DS_AZ_APPLICATION_NAME",
	b"ATTm591641" : "MS_DS_AZ_APPLICATION_VERSION",
	b"ATTm591625" : "MS_DS_AZ_BIZ_RULE",
	b"ATTm591626" : "MS_DS_AZ_BIZ_RULE_LANGUAGE",
	b"ATTm591640" : "MS_DS_AZ_CLASS_ID",
	b"ATTj591619" : "MS_DS_AZ_DOMAIN_TIMEOUT",
	b"ATTi591629" : "MS_DS_AZ_GENERATE_AUDITS",
	b"ATTm591627" : "MS_DS_AZ_LAST_IMPORTED_BIZ_RULE_PATH",
	b"ATTm591616" : "MS_DS_AZ_LDAP_QUERY",
	b"ATTj591648" : "MS_DS_AZ_MAJOR_VERSION",
	b"ATTj591649" : "MS_DS_AZ_MINOR_VERSION",
	b"ATTj591624" : "MS_DS_AZ_OPERATION_ID",
	b"ATTm591623" : "MS_DS_AZ_SCOPE_NAME",
	b"ATTj591620" : "MS_DS_AZ_SCRIPT_ENGINE_CACHE_MAX",
	b"ATTj591621" : "MS_DS_AZ_SCRIPT_TIMEOUT",
	b"ATTi591642" : "MS_DS_AZ_TASK_IS_ROLE_DEFINITION",
	b"ATTj591283" : "MS_DS_BEHAVIOR_VERSION",
	b"ATTk591655" : "MS_DS_BYTE_ARRAY",
	b"ATTk591265" : "MS_DS_CACHED_MEMBERSHIP",
	b"ATTq591266" : "MS_DS_CACHED_MEMBERSHIP_TIME_STAMP",
	b"ATTk591184" : "MS_DS_CONSISTENCY_GUID",
	b"ATTj591185" : "MS_DS_CONSISTENCY_CHILD_COUNT",
	b"ATTr591234" : "MS_DS_CREATOR_SID",
	b"ATTl591656" : "MS_DS_DATE_TIME",
	b"ATTj591670" : "MS_DS_DEFAULT_QUOTA",
	b"ATTm591543" : "MS_DS_DNSROOTALIAS",
	b"ATTl591446" : "MS_DS_ENTRY_TIME_TO_DIE",
	b"ATTk591607" : "MS_DS_EXECUTESCRIPTPASSWORD",
	b"ATTm591657" : "MS_DS_EXTERNAL_KEY",
	b"ATTm591658" : "MS_DS_EXTERNAL_STORE",
	b"ATTm591527" : "MS_DS_FILTER_CONTAINERS",
	b"ATTh591533" : "MS_DS_HAS_INSTANTIATED_NCS",
	b"ATTb591644" : "MS_DS_HAS_DOMAIN_NCS",
	b"ATTb591660" : "MS_DS_HAS_MASTER_NCS",
	b"ATTj591659" : "MS_DS_INTEGER",
	b"ATTj591540" : "MS_DS_INTID",
	b"ATTj591606" : "MS_DS_KEYVERSIONNUMBER",
	b"ATTj591608" : "MS_DS_LOGON_TIME_SYNC_INTERVAL",
	b"ATTb591661" : "MS_DS_MASTERED_BY",
	b"ATTj591666" : "MS_DS_MAX_VALUES",
	b"ATTb591630" : "MS_DS_MEMBERS_FOR_AZ_ROLE",
	b"ATTb591631" : "MS_DS_MEMBERS_FOR_AZ_ROLE_BL",
	b"ATTb591617" : "MS_DS_NON_MEMBERS",
	b"ATTb591618" : "MS_DS_NON_MEMBERS_BL",
	b"ATTk591526" : "MS_DS_TRUST_FOREST_TRUST_INFO",
	b"ATTj591671" : "MS_DS_TOMBSTONE_QUOTA_FACTOR",
	b"ATTm591674" : "MS_DS_TOP_QUOTA_USAGE",
	b"ATTj591235" : "MS_DS_MACHINE_ACCOUNT_QUOTA",
	b"ATTb591664" : "MS_DS_OBJECT_REFERENCE",
	b"ATTb591665" : "MS_DS_OBJECT_REFERENCE_BL",
	b"ATTb591636" : "MS_DS_OPERATIONS_FOR_AZ_ROLE",
	b"ATTb591637" : "MS_DS_OPERATIONS_FOR_AZ_ROLE_BL",
	b"ATTb591632" : "MS_DS_OPERATIONS_FOR_AZ_TASK",
	b"ATTb591633" : "MS_DS_OPERATIONS_FOR_AZ_TASK_BL",
	b"ATTm591445" : "MS_DS_OTHER_SETTINGS",
	b"ATTj591669" : "MS_DS_QUOTA_AMOUNT",
	b"ATTj591672" : "MS_DS_QUOTA_EFFECTIVE",
	b"ATTr591668" : "MS_DS_QUOTA_TRUSTEE",
	b"ATTj591673" : "MS_DS_QUOTA_USED",
	b"ATTm591528" : "MS_DS_NC_REPL_CURSORS",
	b"ATTm591529" : "MS_DS_NC_REPL_INBOUND_NEIGHBORS",
	b"ATTm591530" : "MS_DS_NC_REPL_OUTBOUND_NEIGHBORS",
	b"ATTb591485" : "MS_DS_NC_REPLICA_LOCATIONS",
	b"ATTm591513" : "MS_DS_NON_SECURITY_GROUP_EXTRA_CLASSES",
	b"ATTj591612" : "MS_DS_PER_USER_TRUST_QUOTA",
	b"ATTj591614" : "MS_DS_PER_USER_TRUST_TOMBSTONES_QUOTA",
	b"ATTb591268" : "MS_DS_PREFERRED_GC_SITE",
	b"ATTm591531" : "MS_DS_REPL_ATTRIBUTE_META_DATA",
	b"ATTm591532" : "MS_DS_REPL_VALUE_META_DATA",
	b"ATTh591232" : "MS_DS_REPLICATES_NC_REASON",
	b"ATTj591487" : "MS_DS_REPLICATION_NOTIFY_FIRST_DSA_DELAY",
	b"ATTj591488" : "MS_DS_REPLICATION_NOTIFY_SUBSEQUENT_DSA_DELAY",
	b"ATTj591544" : "MS_DS_REPLICATIONEPOCH",
	b"ATTk591650" : "MS_DS_RETIRED_REPL_NC_SIGNATURES",
	b"ATTk591264" : "MS_DS_SCHEMA_EXTENSIONS",
	b"ATTb591535" : "MS_DS_SD_REFERENCE_DOMAIN",
	b"ATTm591512" : "MS_DS_SECURITY_GROUP_EXTRA_CLASSES",
	b"ATTm591521" : "MS_DS_SETTINGS",
	b"ATTk591267" : "MS_DS_SITE_AFFINITY",
	b"ATTm591539" : "MS_DS_SPN_SUFFIXES",
	b"ATTb591638" : "MS_DS_TASKS_FOR_AZ_ROLE",
	b"ATTb591639" : "MS_DS_TASKS_FOR_AZ_ROLE_BL",
	b"ATTb591634" : "MS_DS_TASKS_FOR_AZ_TASK",
	b"ATTb591635" : "MS_DS_TASKS_FOR_AZ_TASK_BL",
	b"ATTj591284" : "MS_DS_USER_ACCOUNT_CONTROL_COMPUTED",
	b"ATTm591545" : "MS_DS_UPDATESCRIPT",
	b"ATTm131516" : "MS_EXCH_ASSISTANT_NAME",
	b"ATTm131668" : "MS_EXCH_HOUSE_IDENTIFIER",
	b"ATTm131665" : "MS_EXCH_LABELEDURI",
	b"ATTb131176" : "MS_EXCH_OWNER_BL",
	b"ATTb591517" : "MS_FRS_HUB_MEMBER",
	b"ATTm591516" : "MS_FRS_TOPOLOGY_PREF",
	b"ATTk591645" : "MS_IEEE_80211_DATA",
	b"ATTj591646" : "MS_IEEE_80211_DATA_TYPE",
	b"ATTm591647" : "MS_IEEE_80211_ID",
	b"ATTm591610" : "MS_IIS_FTP_DIR",
	b"ATTm591609" : "MS_IIS_FTP_ROOT",
	b"ATTm591260" : "MS_PKI_CERT_TEMPLATE_OID",
	b"ATTm591498" : "MS_PKI_CERTIFICATE_APPLICATION_POLICY",
	b"ATTj591256" : "MS_PKI_CERTIFICATE_NAME_FLAG",
	b"ATTm591263" : "MS_PKI_CERTIFICATE_POLICY",
	b"ATTj591254" : "MS_PKI_ENROLLMENT_FLAG",
	b"ATTj591257" : "MS_PKI_MINIMAL_KEY_SIZE",
	b"ATTj591495" : "MS_PKI_OID_ATTRIBUTE",
	b"ATTm591496" : "MS_PKI_OID_CPS",
	b"ATTm591536" : "MS_PKI_OID_LOCALIZEDNAME",
	b"ATTm591497" : "MS_PKI_OID_USER_NOTICE",
	b"ATTj591255" : "MS_PKI_PRIVATE_KEY_FLAG",
	b"ATTm591261" : "MS_PKI_SUPERSEDE_TEMPLATES",
	b"ATTj591259" : "MS_PKI_TEMPLATE_MINOR_REVISION",
	b"ATTj591258" : "MS_PKI_TEMPLATE_SCHEMA_VERSION",
	b"ATTm591499" : "MS_PKI_RA_APPLICATION_POLICIES",
	b"ATTm591262" : "MS_PKI_RA_POLICIES",
	b"ATTj591253" : "MS_PKI_RA_SIGNATURE",
	b"ATTm590708" : "MS_RRAS_ATTRIBUTE",
	b"ATTm590707" : "MS_RRAS_VENDOR_ATTRIBUTE_ENTRY",
	b"ATTm591187" : "MS_SQL_NAME",
	b"ATTm591188" : "MS_SQL_REGISTEREDOWNER",
	b"ATTm591189" : "MS_SQL_CONTACT",
	b"ATTm591190" : "MS_SQL_LOCATION",
	b"ATTq591191" : "MS_SQL_MEMORY",
	b"ATTj591192" : "MS_SQL_BUILD",
	b"ATTm591193" : "MS_SQL_SERVICEACCOUNT",
	b"ATTj591194" : "MS_SQL_CHARACTERSET",
	b"ATTm591195" : "MS_SQL_SORTORDER",
	b"ATTj591196" : "MS_SQL_UNICODESORTORDER",
	b"ATTi591197" : "MS_SQL_CLUSTERED",
	b"ATTm591198" : "MS_SQL_NAMEDPIPE",
	b"ATTm591199" : "MS_SQL_MULTIPROTOCOL",
	b"ATTm591200" : "MS_SQL_SPX",
	b"ATTm591201" : "MS_SQL_TCPIP",
	b"ATTm591202" : "MS_SQL_APPLETALK",
	b"ATTm591203" : "MS_SQL_VINES",
	b"ATTq591204" : "MS_SQL_STATUS",
	b"ATTm591205" : "MS_SQL_LASTUPDATEDDATE",
	b"ATTm591206" : "MS_SQL_INFORMATIONURL",
	b"ATTm591207" : "MS_SQL_CONNECTIONURL",
	b"ATTm591208" : "MS_SQL_PUBLICATIONURL",
	b"ATTm591209" : "MS_SQL_GPSLATITUDE",
	b"ATTm591210" : "MS_SQL_GPSLONGITUDE",
	b"ATTm591211" : "MS_SQL_GPSHEIGHT",
	b"ATTm591212" : "MS_SQL_VERSION",
	b"ATTm591213" : "MS_SQL_LANGUAGE",
	b"ATTm591214" : "MS_SQL_DESCRIPTION",
	b"ATTm591215" : "MS_SQL_TYPE",
	b"ATTi591216" : "MS_SQL_INFORMATIONDIRECTORY",
	b"ATTm591217" : "MS_SQL_DATABASE",
	b"ATTi591218" : "MS_SQL_ALLOWANONYMOUSSUBSCRIPTION",
	b"ATTm591219" : "MS_SQL_ALIAS",
	b"ATTq591220" : "MS_SQL_SIZE",
	b"ATTm591221" : "MS_SQL_CREATIONDATE",
	b"ATTm591222" : "MS_SQL_LASTBACKUPDATE",
	b"ATTm591223" : "MS_SQL_LASTDIAGNOSTICDATE",
	b"ATTm591224" : "MS_SQL_APPLICATIONS",
	b"ATTm591225" : "MS_SQL_KEYWORDS",
	b"ATTm591226" : "MS_SQL_PUBLISHER",
	b"ATTi591227" : "MS_SQL_ALLOWKNOWNPULLSUBSCRIPTION",
	b"ATTi591228" : "MS_SQL_ALLOWIMMEDIATEUPDATINGSUBSCRIPTION",
	b"ATTi591229" : "MS_SQL_ALLOWQUEUEDUPDATINGSUBSCRIPTION",
	b"ATTi591230" : "MS_SQL_ALLOWSNAPSHOTFILESFTPDOWNLOADING",
	b"ATTi591231" : "MS_SQL_THIRDPARTY",
	b"ATTk591524" : "MS_TAPI_CONFERENCE_BLOB",
	b"ATTm591525" : "MS_TAPI_IP_ADDRESS",
	b"ATTm591523" : "MS_TAPI_PROTOCOL_ID",
	b"ATTm591522" : "MS_TAPI_UNIQUE_IDENTIFIER",
	b"ATTm591447" : "MS_WMI_AUTHOR",
	b"ATTm591448" : "MS_WMI_CHANGEDATE",
	b"ATTm591500" : "MS_WMI_CLASS",
	b"ATTm591449" : "MS_WMI_CLASSDEFINITION",
	b"ATTm591450" : "MS_WMI_CREATIONDATE",
	b"ATTj591501" : "MS_WMI_GENUS",
	b"ATTm591451" : "MS_WMI_ID",
	b"ATTj591452" : "MS_WMI_INTDEFAULT",
	b"ATTj591502" : "MS_WMI_INTFLAGS1",
	b"ATTj591503" : "MS_WMI_INTFLAGS2",
	b"ATTj591504" : "MS_WMI_INTFLAGS3",
	b"ATTj591505" : "MS_WMI_INTFLAGS4",
	b"ATTj591453" : "MS_WMI_INTMAX",
	b"ATTj591454" : "MS_WMI_INTMIN",
	b"ATTj591455" : "MS_WMI_INTVALIDVALUES",
	b"ATTq591456" : "MS_WMI_INT8DEFAULT",
	b"ATTq591457" : "MS_WMI_INT8MAX",
	b"ATTq591458" : "MS_WMI_INT8MIN",
	b"ATTq591459" : "MS_WMI_INT8VALIDVALUES",
	b"ATTm591462" : "MS_WMI_MOF",
	b"ATTm591463" : "MS_WMI_NAME",
	b"ATTm591464" : "MS_WMI_NORMALIZEDCLASS",
	b"ATTm591506" : "MS_WMI_PARM1",
	b"ATTm591507" : "MS_WMI_PARM2",
	b"ATTm591508" : "MS_WMI_PARM3",
	b"ATTm591509" : "MS_WMI_PARM4",
	b"ATTm591465" : "MS_WMI_PROPERTYNAME",
	b"ATTm591466" : "MS_WMI_QUERY",
	b"ATTm591467" : "MS_WMI_QUERYLANGUAGE",
	b"ATTm591510" : "MS_WMI_SCOPEGUID",
	b"ATTm591468" : "MS_WMI_SOURCEORGANIZATION",
	b"ATTm591460" : "MS_WMI_STRINGDEFAULT",
	b"ATTm591461" : "MS_WMI_STRINGVALIDVALUES",
	b"ATTm591469" : "MS_WMI_TARGETCLASS",
	b"ATTm591470" : "MS_WMI_TARGETNAMESPACE",
	b"ATTk591471" : "MS_WMI_TARGETOBJECT",
	b"ATTm591472" : "MS_WMI_TARGETPATH",
	b"ATTm591473" : "MS_WMI_TARGETTYPE",
	b"ATTf590540" : "MSCOPE_ID",
	b"ATTm590495" : "MSI_FILE_LIST",
	b"ATTk590638" : "MSI_SCRIPT",
	b"ATTm590669" : "MSI_SCRIPT_NAME",
	b"ATTm589839" : "MSI_SCRIPT_PATH",
	b"ATTj590670" : "MSI_SCRIPT_SIZE",
	b"ATTi590747" : "MSMQ_AUTHENTICATE",
	b"ATTj590744" : "MSMQ_BASE_PRIORITY",
	b"ATTe590757" : "MSMQ_COMPUTER_TYPE",
	b"ATTm591241" : "MSMQ_COMPUTER_TYPE_EX",
	b"ATTj590770" : "MSMQ_COST",
	b"ATTe590764" : "MSMQ_CSP_NAME",
	b"ATTi591063" : "MSMQ_DEPENDENT_CLIENT_SERVICE",
	b"ATTi591050" : "MSMQ_DEPENDENT_CLIENT_SERVICES",
	b"ATTk590772" : "MSMQ_DIGESTS",
	b"ATTk590790" : "MSMQ_DIGESTS_MIG",
	b"ATTi591062" : "MSMQ_DS_SERVICE",
	b"ATTi591052" : "MSMQ_DS_SERVICES",
	b"ATTk590760" : "MSMQ_ENCRYPT_KEY",
	b"ATTi590758" : "MSMQ_FOREIGN",
	b"ATTb590753" : "MSMQ_IN_ROUTING_SERVERS",
	b"ATTj591132" : "MSMQ_INTERVAL1",
	b"ATTj591133" : "MSMQ_INTERVAL2",
	b"ATTi590742" : "MSMQ_JOURNAL",
	b"ATTj590745" : "MSMQ_JOURNAL_QUOTA",
	b"ATTe590746" : "MSMQ_LABEL",
	b"ATTm591239" : "MSMQ_LABEL_EX",
	b"ATTj590765" : "MSMQ_LONG_LIVED",
	b"ATTi590776" : "MSMQ_MIGRATED",
	b"ATTm591538" : "MSMQ_MULTICAST_ADDRESS",
	b"ATTi590763" : "MSMQ_NAME_STYLE",
	b"ATTj590788" : "MSMQ_NT4_FLAGS",
	b"ATTj590784" : "MSMQ_NT4_STUB",
	b"ATTj590759" : "MSMQ_OS_TYPE",
	b"ATTb590752" : "MSMQ_OUT_ROUTING_SERVERS",
	b"ATTk590749" : "MSMQ_OWNER_ID",
	b"ATTb591049" : "MSMQ_PREV_SITE_GATES",
	b"ATTj590748" : "MSMQ_PRIVACY_LEVEL",
	b"ATTk590775" : "MSMQ_QM_ID",
	b"ATTj590787" : "MSMQ_QUEUE_JOURNAL_QUOTA",
	b"ATTm591067" : "MSMQ_QUEUE_NAME_EXT",
	b"ATTj590786" : "MSMQ_QUEUE_QUOTA",
	b"ATTk590741" : "MSMQ_QUEUE_TYPE",
	b"ATTj590743" : "MSMQ_QUOTA",
	b"ATTm591519" : "MSMQ_RECIPIENT_FORMATNAME",
	b"ATTi591061" : "MSMQ_ROUTING_SERVICE",
	b"ATTi591051" : "MSMQ_ROUTING_SERVICES",
	b"ATTi591537" : "MSMQ_SECURED_SOURCE",
	b"ATTj590754" : "MSMQ_SERVICE_TYPE",
	b"ATTj590774" : "MSMQ_SERVICES",
	b"ATTk590771" : "MSMQ_SIGN_CERTIFICATES",
	b"ATTk590791" : "MSMQ_SIGN_CERTIFICATES_MIG",
	b"ATTk590761" : "MSMQ_SIGN_KEY",
	b"ATTb590767" : "MSMQ_SITE_1",
	b"ATTb590768" : "MSMQ_SITE_2",
	b"ATTi590785" : "MSMQ_SITE_FOREIGN",
	b"ATTb590769" : "MSMQ_SITE_GATES",
	b"ATTb591134" : "MSMQ_SITE_GATES_MIG",
	b"ATTk590777" : "MSMQ_SITE_ID",
	b"ATTe590789" : "MSMQ_SITE_NAME",
	b"ATTm591240" : "MSMQ_SITE_NAME_EX",
	b"ATTk590751" : "MSMQ_SITES",
	b"ATTi590750" : "MSMQ_TRANSACTIONAL",
	b"ATTk591161" : "MSMQ_USER_SID",
	b"ATTj590766" : "MSMQ_VERSION",
	b"ATTi590943" : "MSNPALLOWDIALIN",
	b"ATTf590947" : "MSNPCALLEDSTATIONID",
	b"ATTf590948" : "MSNPCALLINGSTATIONID",
	b"ATTf590954" : "MSNPSAVEDCALLINGSTATIONID",
	b"ATTf590969" : "MSRADIUSCALLBACKNUMBER",
	b"ATTj590977" : "MSRADIUSFRAMEDIPADDRESS",
	b"ATTf590982" : "MSRADIUSFRAMEDROUTE",
	b"ATTj590995" : "MSRADIUSSERVICETYPE",
	b"ATTf591013" : "MSRASSAVEDCALLBACKNUMBER",
	b"ATTj591014" : "MSRASSAVEDFRAMEDIPADDRESS",
	b"ATTf591015" : "MSRASSAVEDFRAMEDROUTE",
	b"ATTc131096" : "MUST_CONTAIN",
	b"ATTj590577" : "NAME_SERVICE_FLAGS",
	b"ATTb131088" : "NC_NAME",
	b"ATTm589911" : "NETBIOS_NAME",
	b"ATTi590673" : "NETBOOT_ALLOW_NEW_CLIENTS",
	b"ATTi590678" : "NETBOOT_ANSWER_ONLY_VALID_CLIENTS",
	b"ATTi590677" : "NETBOOT_ANSWER_REQUESTS",
	b"ATTj590676" : "NETBOOT_CURRENT_CLIENT_COUNT",
	b"ATTk590183" : "NETBOOT_GUID",
	b"ATTm590182" : "NETBOOT_INITIALIZATION",
	b"ATTm590681" : "NETBOOT_INTELLIMIRROR_OSES",
	b"ATTi590674" : "NETBOOT_LIMIT_CLIENTS",
	b"ATTm590683" : "NETBOOT_LOCALLY_INSTALLED_OSES",
	b"ATTm590185" : "NETBOOT_MACHINE_FILE_PATH",
	b"ATTj590675" : "NETBOOT_MAX_CLIENTS",
	b"ATTm591065" : "NETBOOT_MIRROR_DATA_FILE",
	b"ATTm590679" : "NETBOOT_NEW_MACHINE_NAMING_POLICY",
	b"ATTb590680" : "NETBOOT_NEW_MACHINE_OU",
	b"ATTb590688" : "NETBOOT_SCP_BL",
	b"ATTb590684" : "NETBOOT_SERVER",
	b"ATTm591064" : "NETBOOT_SIF_FILE",
	b"ATTm590682" : "NETBOOT_TOOLS",
	b"ATTe131531" : "NETWORK_ADDRESS",
	b"ATTb590038" : "NEXT_LEVEL_STORE",
	b"ATTj589912" : "NEXT_RID",
	b"ATTb590354" : "NON_SECURITY_MEMBER",
	b"ATTb590355" : "NON_SECURITY_MEMBER_BL",
	b"ATTb590127" : "NOTIFICATION_LIST",
	b"ATTk589913" : "NT_GROUP_MEMBERS",
	b"ATTj590181" : "NT_MIXED_DOMAIN",
	b"ATTk589918" : "NT_PWD_HISTORY",
	b"ATTp131353" : "NT_SECURITY_DESCRIPTOR",
	b"ATTb49" : "OBJ_DIST_NAME",
	b"ATTb590606" : "OBJECT_CATEGORY",
	b"ATTc0" : "OBJECT_CLASS",
	b"ATTj131442" : "OBJECT_CLASS_CATEGORY",
	b"ATTm1572870" : "OBJECT_CLASSES",
	b"ATTj590330" : "OBJECT_COUNT",
	b"ATTk589826" : "OBJECT_GUID",
	b"ATTr589970" : "OBJECT_SID",
	b"ATTj131148" : "OBJECT_VERSION",
	b"ATTm589975" : "OEM_INFORMATION",
	b"ATTk131290" : "OM_OBJECT_CLASS",
	b"ATTj131303" : "OM_SYNTAX",
	b"ATTk590329" : "OMT_GUID",
	b"ATTk590157" : "OMT_INDX_GUID",
	b"ATTm590187" : "OPERATING_SYSTEM",
	b"ATTm590239" : "OPERATING_SYSTEM_HOTFIX",
	b"ATTm590189" : "OPERATING_SYSTEM_SERVICE_PACK",
	b"ATTm590188" : "OPERATING_SYSTEM_VERSION",
	b"ATTj589968" : "OPERATOR_COUNT",
	b"ATTm590536" : "OPTION_DESCRIPTION",
	b"ATTj590131" : "OPTIONS",
	b"ATTf590537" : "OPTIONS_LOCATION",
	b"ATTm10" : "ORGANIZATION_NAME",
	b"ATTm11" : "ORGANIZATIONAL_UNIT_NAME",
	b"ATTm1376301" : "ORGANIZATIONALSTATUS",
	b"ATTk131517" : "ORIGINAL_DISPLAY_TABLE",
	b"ATTk131286" : "ORIGINAL_DISPLAY_TABLE_MSDOS",
	b"ATTm589915" : "OTHER_LOGIN_WORKSTATIONS",
	b"ATTm590475" : "OTHER_MAILBOX",
	b"ATTm1441826" : "OTHER_NAME",
	b"ATTh591183" : "OTHER_WELL_KNOWN_OBJECTS",
	b"ATTb32" : "OWNER",
	b"ATTj590151" : "PACKAGE_FLAGS",
	b"ATTm590150" : "PACKAGE_NAME",
	b"ATTj590148" : "PACKAGE_TYPE",
	b"ATTb590381" : "PARENT_CA",
	b"ATTk590509" : "PARENT_CA_CERTIFICATE_CHAIN",
	b"ATTk591048" : "PARENT_GUID",
	b"ATTk590487" : "PARTIAL_ATTRIBUTE_DELETION_LIST",
	b"ATTk590464" : "PARTIAL_ATTRIBUTE_SET",
	b"ATTq590690" : "PEK_KEY_CHANGE_INTERVAL",
	b"ATTk590689" : "PEK_LIST",
	b"ATTk590517" : "PENDING_CA_CERTIFICATES",
	b"ATTb590519" : "PENDING_PARENT_CA",
	b"ATTk131397" : "PER_MSG_DIALOG_DISPLAY_TABLE",
	b"ATTk131398" : "PER_RECIP_DIALOG_DISPLAY_TABLE",
	b"ATTm131687" : "PERSONAL_TITLE",
	b"ATTm590470" : "PHONE_FAX_OTHER",
	b"ATTm131349" : "PHONE_HOME_OTHER",
	b"ATTm1376276" : "PHONE_HOME_PRIMARY",
	b"ATTm590546" : "PHONE_IP_OTHER",
	b"ATTm590545" : "PHONE_IP_PRIMARY",
	b"ATTm590473" : "PHONE_ISDN_PRIMARY",
	b"ATTm590471" : "PHONE_MOBILE_OTHER",
	b"ATTm1376297" : "PHONE_MOBILE_PRIMARY",
	b"ATTm131090" : "PHONE_OFFICE_OTHER",
	b"ATTm131190" : "PHONE_PAGER_OTHER",
	b"ATTm1376298" : "PHONE_PAGER_PRIMARY",
	b"ATTk1376263" : "PHOTO",
	b"ATTm19" : "PHYSICAL_DELIVERY_OFFICE_NAME",
	b"ATTb590338" : "PHYSICAL_LOCATION_OBJECT",
	b"ATTk1441827" : "PICTURE",
	b"ATTm591154" : "PKI_CRITICAL_EXTENSIONS",
	b"ATTm591158" : "PKI_DEFAULT_CSPS",
	b"ATTj591151" : "PKI_DEFAULT_KEY_SPEC",
	b"ATTp591159" : "PKI_ENROLLMENT_ACCESS",
	b"ATTk591155" : "PKI_EXPIRATION_PERIOD",
	b"ATTm591157" : "PKI_EXTENDED_KEY_USAGE",
	b"ATTk591152" : "PKI_KEY_USAGE",
	b"ATTj591153" : "PKI_MAX_ISSUING_DEPTH",
	b"ATTk591156" : "PKI_OVERLAP_PERIOD",
	b"ATTk590030" : "PKT",
	b"ATTk590029" : "PKT_GUID",
	b"ATTj590457" : "POLICY_REPLICATION_FLAGS",
	b"ATTm590052" : "PORT_NAME",
	b"ATTc131080" : "POSS_SUPERIORS",
	b"ATTc590739" : "POSSIBLE_INFERIORS",
	b"ATTm18" : "POST_OFFICE_BOX",
	b"ATTm16" : "POSTAL_ADDRESS",
	b"ATTm17" : "POSTAL_CODE",
	b"ATTj28" : "PREFERRED_DELIVERY_METHOD",
	b"ATTm1441831" : "PREFERREDLANGUAGE",
	b"ATTb589921" : "PREFERRED_OU",
	b"ATTk590362" : "PREFIX_MAP",
	b"ATTn29" : "PRESENTATION_ADDRESS",
	b"ATTk590516" : "PREVIOUS_CA_CERTIFICATES",
	b"ATTb590518" : "PREVIOUS_PARENT_CA",
	b"ATTj589922" : "PRIMARY_GROUP_ID",
	b"ATTj591236" : "PRIMARY_GROUP_TOKEN",
	b"ATTj590071" : "PRINT_ATTRIBUTES",
	b"ATTm590061" : "PRINT_BIN_NAMES",
	b"ATTi590066" : "PRINT_COLLATE",
	b"ATTi590067" : "PRINT_COLOR",
	b"ATTi591135" : "PRINT_DUPLEX_SUPPORTED",
	b"ATTj590058" : "PRINT_END_TIME",
	b"ATTm590059" : "PRINT_FORM_NAME",
	b"ATTi590099" : "PRINT_KEEP_PRINTED_JOBS",
	b"ATTm590070" : "PRINT_LANGUAGE",
	b"ATTm590112" : "PRINT_MAC_ADDRESS",
	b"ATTj590065" : "PRINT_MAX_COPIES",
	b"ATTj590062" : "PRINT_MAX_RESOLUTION_SUPPORTED",
	b"ATTj590101" : "PRINT_MAX_X_EXTENT",
	b"ATTj590102" : "PRINT_MAX_Y_EXTENT",
	b"ATTm590113" : "PRINT_MEDIA_READY",
	b"ATTm590123" : "PRINT_MEDIA_SUPPORTED",
	b"ATTj590106" : "PRINT_MEMORY",
	b"ATTj590103" : "PRINT_MIN_X_EXTENT",
	b"ATTj590104" : "PRINT_MIN_Y_EXTENT",
	b"ATTm590111" : "PRINT_NETWORK_ADDRESS",
	b"ATTm590096" : "PRINT_NOTIFY",
	b"ATTj590114" : "PRINT_NUMBER_UP",
	b"ATTm590064" : "PRINT_ORIENTATIONS_SUPPORTED",
	b"ATTm590095" : "PRINT_OWNER",
	b"ATTj590455" : "PRINT_PAGES_PER_MINUTE",
	b"ATTj590109" : "PRINT_RATE",
	b"ATTm590110" : "PRINT_RATE_UNIT",
	b"ATTm590054" : "PRINT_SEPARATOR_FILE",
	b"ATTm590094" : "PRINT_SHARE_NAME",
	b"ATTm590098" : "PRINT_SPOOLING",
	b"ATTi590105" : "PRINT_STAPLING_SUPPORTED",
	b"ATTj590057" : "PRINT_START_TIME",
	b"ATTm590097" : "PRINT_STATUS",
	b"ATTm590124" : "PRINTER_NAME",
	b"ATTq589923" : "PRIOR_SET_TIME",
	b"ATTk589924" : "PRIOR_VALUE",
	b"ATTj590055" : "PRIORITY",
	b"ATTk589925" : "PRIVATE_KEY",
	b"ATTj590460" : "PRIVILEGE_ATTRIBUTES",
	b"ATTm590458" : "PRIVILEGE_DISPLAY_NAME",
	b"ATTb590461" : "PRIVILEGE_HOLDER",
	b"ATTq590459" : "PRIVILEGE_VALUE",
	b"ATTk590642" : "PRODUCT_CODE",
	b"ATTm589963" : "PROFILE_PATH",
	b"ATTh591073" : "PROXIED_OBJECT_NAME",
	b"ATTm131282" : "PROXY_ADDRESSES",
	b"ATTi131595" : "PROXY_GENERATION_ENABLED",
	b"ATTq589927" : "PROXY_LIFETIME",
	b"ATTk590244" : "PUBLIC_KEY_POLICY",
	b"ATTm590710" : "PURPORTED_SEARCH",
	b"ATTj589919" : "PWD_HISTORY_LENGTH",
	b"ATTq589920" : "PWD_LAST_SET",
	b"ATTj589917" : "PWD_PROPERTIES",
	b"ATTj590282" : "QUALITY_OF_SERVICE",
	b"ATTm591179" : "QUERY_FILTER",
	b"ATTb590432" : "QUERY_POLICY_BL",
	b"ATTb590431" : "QUERY_POLICY_OBJECT",
	b"ATTm590504" : "QUERYPOINT",
	b"ATTj131106" : "RANGE_LOWER",
	b"ATTj131107" : "RANGE_UPPER",
	b"ATTm589825" : "RDN",
	b"ATTc131098" : "RDN_ID",
	b"ATTk26" : "REGISTERED_ADDRESS",
	b"ATTm589929" : "REMOTE_SERVER_NAME",
	b"ATTm589931" : "REMOTE_SOURCE",
	b"ATTj589932" : "REMOTE_SOURCE_TYPE",
	b"ATTm590633" : "REMOTE_STORAGE_GUID",
	b"ATTk589827" : "REPL_PROPERTY_META_DATA",
	b"ATTj590501" : "REPL_TOPOLOGY_STAY_OF_EXECUTION",
	b"ATTk589828" : "REPL_UPTODATE_VECTOR",
	b"ATTm589933" : "REPLICA_SOURCE",
	b"ATTb131508" : "REPORTS",
	b"ATTj591160" : "REPL_INTERVAL",
	b"ATTk131163" : "REPS_FROM",
	b"ATTk131155" : "REPS_TO",
	b"ATTk590145" : "REQUIRED_CATEGORIES",
	b"ATTk590497" : "RETIRED_REPL_DSA_SIGNATURES",
	b"ATTr591125" : "TOKEN_GROUPS",
	b"ATTr591242" : "TOKEN_GROUPS_GLOBAL_AND_UNIVERSAL",
	b"ATTr591127" : "TOKEN_GROUPS_NO_GC_ACCEPTABLE",
	b"ATTj589969" : "REVISION",
	b"ATTj589977" : "RID",
	b"ATTq590195" : "RID_ALLOCATION_POOL",
	b"ATTq590194" : "RID_AVAILABLE_POOL",
	b"ATTb590192" : "RID_MANAGER_REFERENCE",
	b"ATTj590198" : "RID_NEXT_RID",
	b"ATTq590196" : "RID_PREVIOUS_ALLOCATION_POOL",
	b"ATTb590493" : "RID_SET_REFERENCES",
	b"ATTq590197" : "RID_USED_POOL",
	b"ATTm590164" : "RIGHTS_GUID",
	b"ATTb33" : "ROLE_OCCUPANT",
	b"ATTm1376262" : "ROOMNUMBER",
	b"ATTb590498" : "ROOT_TRUST",
	b"ATTm590190" : "RPC_NS_ANNOTATION",
	b"ATTm589937" : "RPC_NS_BINDINGS",
	b"ATTm590191" : "RPC_NS_CODESET",
	b"ATTj590578" : "RPC_NS_ENTRY_FLAGS",
	b"ATTm589938" : "RPC_NS_GROUP",
	b"ATTm589939" : "RPC_NS_INTERFACE_ID",
	b"ATTm590136" : "RPC_NS_OBJECT_ID",
	b"ATTj589941" : "RPC_NS_PRIORITY",
	b"ATTm589942" : "RPC_NS_PROFILE_ENTRY",
	b"ATTm590138" : "RPC_NS_TRANSFER_SYNTAX",
	b"ATTm590045" : "SAM_ACCOUNT_NAME",
	b"ATTj590126" : "SAM_ACCOUNT_TYPE",
	b"ATTk590035" : "SCHEDULE",
	b"ATTj589944" : "SCHEMA_FLAGS_EX",
	b"ATTk589972" : "SCHEMA_ID_GUID",
	b"ATTk591182" : "SCHEMA_INFO",
	b"ATTl590305" : "SCHEMA_UPDATE",
	b"ATTj131543" : "SCHEMA_VERSION",
	b"ATTj591178" : "SCOPE_FLAGS",
	b"ATTm589886" : "SCRIPT_PATH",
	b"ATTj591128" : "SD_RIGHTS_EFFECTIVE",
	b"ATTj131406" : "SEARCH_FLAGS",
	b"ATTk14" : "SEARCH_GUIDE",
	b"ATTb1376277" : "SECRETARY",
	b"ATTr589945" : "SECURITY_IDENTIFIER",
	b"ATTb34" : "SEE_ALSO",
	b"ATTj590328" : "SEQ_NOTIFICATION",
	b"ATTf5" : "SERIAL_NUMBER",
	b"ATTm590047" : "SERVER_NAME",
	b"ATTb590339" : "SERVER_REFERENCE",
	b"ATTb590340" : "SERVER_REFERENCE_BL",
	b"ATTj589981" : "SERVER_ROLE",
	b"ATTj589978" : "SERVER_STATE",
	b"ATTm590334" : "SERVICE_BINDING_INFORMATION",
	b"ATTk589946" : "SERVICE_CLASS_ID",
	b"ATTk589947" : "SERVICE_CLASS_INFO",
	b"ATTm590333" : "SERVICE_CLASS_NAME",
	b"ATTm590481" : "SERVICE_DNS_NAME",
	b"ATTm590483" : "SERVICE_DNS_NAME_TYPE",
	b"ATTk590023" : "SERVICE_INSTANCE_VERSION",
	b"ATTm590595" : "SERVICE_PRINCIPAL_NAME",
	b"ATTm590149" : "SETUP_COMMAND",
	b"ATTm590439" : "SHELL_CONTEXT_MENU",
	b"ATTm590387" : "SHELL_PROPERTY_PAGES",
	b"ATTm591033" : "SHORT_SERVER_NAME",
	b"ATTb590468" : "SHOW_IN_ADDRESS_BOOK",
	b"ATTi131241" : "SHOW_IN_ADVANCED_VIEW_ONLY",
	b"ATTr590433" : "SID_HISTORY",
	b"ATTm590648" : "SIGNATURE_ALGORITHMS",
	b"ATTk590186" : "SITE_GUID",
	b"ATTb590646" : "SITE_LINK_LIST",
	b"ATTb590645" : "SITE_LIST",
	b"ATTb590336" : "SITE_OBJECT",
	b"ATTb590337" : "SITE_OBJECT_BL",
	b"ATTb590318" : "SITE_SERVER",
	b"ATTm590610" : "SMTP_MAIL_ADDRESS",
	b"ATTm591171" : "SPN_MAPPINGS",
	b"ATTm8" : "STATE_OR_PROVINCE_NAME",
	b"ATTm9" : "STREET_ADDRESS",
	b"ATTc1572873" : "STRUCTURAL_OBJECT_CLASS",
	b"ATTc131093" : "SUB_CLASS_OF",
	b"ATTb131079" : "SUB_REFS",
	b"ATTb1638410" : "SUBSCHEMASUBENTRY",
	b"ATTm590535" : "SUPER_SCOPE_DESCRIPTION",
	b"ATTf590534" : "SUPER_SCOPES",
	b"ATTm590356" : "SUPERIOR_DNS_ROOT",
	b"ATTk589949" : "SUPPLEMENTAL_CREDENTIALS",
	b"ATTk30" : "SUPPORTED_APPLICATION_CONTEXT",
	b"ATTm4" : "SURNAME",
	b"ATTj590490" : "SYNC_ATTRIBUTES",
	b"ATTb590489" : "SYNC_MEMBERSHIP",
	b"ATTb590488" : "SYNC_WITH_OBJECT",
	b"ATTr590491" : "SYNC_WITH_SID",
	b"ATTc590022" : "SYSTEM_AUXILIARY_CLASS",
	b"ATTj590199" : "SYSTEM_FLAGS",
	b"ATTc590020" : "SYSTEM_MAY_CONTAIN",
	b"ATTc590021" : "SYSTEM_MUST_CONTAIN",
	b"ATTi589994" : "SYSTEM_ONLY",
	b"ATTc590019" : "SYSTEM_POSS_SUPERIORS",
	b"ATTm20" : "TELEPHONE_NUMBER",
	b"ATTk22" : "TELETEX_TERMINAL_IDENTIFIER",
	b"ATTk21" : "TELEX_NUMBER",
	b"ATTm590472" : "TELEX_PRIMARY",
	b"ATTb591170" : "TEMPLATE_ROOTS",
	b"ATTk590709" : "TERMINAL_SERVER",
	b"ATTm131203" : "TEXT_COUNTRY",
	b"ATTm1376258" : "TEXT_ENCODED_OR_ADDRESS",
	b"ATTq590327" : "TIME_REFRESH",
	b"ATTq590326" : "TIME_VOL_CHANGE",
	b"ATTm12" : "TITLE",
	b"ATTj131126" : "TOMBSTONE_LIFETIME",
	b"ATTc590719" : "TRANSPORT_ADDRESS_ATTRIBUTE",
	b"ATTm590613" : "TRANSPORT_DLL_NAME",
	b"ATTb590615" : "TRANSPORT_TYPE",
	b"ATTi590630" : "TREAT_AS_LEAF",
	b"ATTm590484" : "TREE_NAME",
	b"ATTj590294" : "TRUST_ATTRIBUTES",
	b"ATTk589953" : "TRUST_AUTH_INCOMING",
	b"ATTk589959" : "TRUST_AUTH_OUTGOING",
	b"ATTj589956" : "TRUST_DIRECTION",
	b"ATTb590295" : "TRUST_PARENT",
	b"ATTm589957" : "TRUST_PARTNER",
	b"ATTj589958" : "TRUST_POSIX_OFFSET",
	b"ATTj589960" : "TRUST_TYPE",
	b"ATTj589979" : "UAS_COMPAT",
	b"ATTm1376257" : "UID",
	b"ATTm589961" : "UNC_NAME",
	b"ATTk589914" : "UNICODE_PWD",
	b"ATTm1376300" : "UNIQUEIDENTIFIER",
	b"ATTb50" : "UNIQUEMEMBER",
	b"ATTk590637" : "UPGRADE_PRODUCT_CODE",
	b"ATTm590714" : "UPN_SUFFIXES",
	b"ATTj589832" : "USER_ACCOUNT_CONTROL",
	b"ATTk590469" : "USER_CERT",
	b"ATTm589980" : "USER_COMMENT",
	b"ATTm589962" : "USER_PARAMETERS",
	b"ATTk35" : "USER_PASSWORD",
	b"ATTm1376264" : "USERCLASS",
	b"ATTk1442008" : "USERPKCS12",
	b"ATTm590480" : "USER_PRINCIPAL_NAME",
	b"ATTm590575" : "USER_SHARED_FOLDER",
	b"ATTm590576" : "USER_SHARED_FOLDER_OTHER",
	b"ATTk1310860" : "USER_SMIME_CERTIFICATE",
	b"ATTm589910" : "USER_WORKSTATIONS",
	b"ATTq131192" : "USN_CHANGED",
	b"ATTq131091" : "USN_CREATED",
	b"ATTq131339" : "USN_DSA_LAST_OBJ_REMOVED",
	b"ATTj131541" : "USN_INTERSITE",
	b"ATTq131193" : "USN_LAST_OBJ_REM",
	b"ATTq590720" : "USN_SOURCE",
	b"ATTj591180" : "VALID_ACCESSES",
	b"ATTm590079" : "VENDOR",
	b"ATTj589965" : "VERSION_NUMBER",
	b"ATTj590152" : "VERSION_NUMBER_HI",
	b"ATTj590153" : "VERSION_NUMBER_LO",
	b"ATTk590160" : "VOL_TABLE_GUID",
	b"ATTk590158" : "VOL_TABLE_IDX_GUID",
	b"ATTj590331" : "VOLUME_COUNT",
	b"ATTm590125" : "WBEM_PATH",
	b"ATTh590442" : "WELL_KNOWN_OBJECTS",
	b"ATTl131075" : "WHEN_CHANGED",
	b"ATTl131074" : "WHEN_CREATED",
	b"ATTk589966" : "WINSOCK_ADDRESSES",
	b"ATTm131536" : "WWW_HOME_PAGE",
	b"ATTm590573" : "WWW_PAGE_OTHER",
	b"ATTg24" : "X121_ADDRESS",
	b"ATTk45" : "X500UNIQUEIDENTIFIER",
	b"ATTk36" : "X509_CERT",
}

COLNAME_TO_ARR = dict((v,k) for k,v in ATTR_TO_COLNAME.items())